@@ -1,13 +1,13 @@
-#' @importFrom utils head tail object.size str
-#' @importFrom stats runif aggregate na.omit setNames
+#' @importFrom utils head object.size str tail
+#' @importFrom stats aggregate na.omit rbinom runif setNames
 #' @importFrom tools file_ext file_path_sans_ext
-#' @importFrom methods as slotNames new slot
-#' @importFrom grid convertUnit current.viewport linesGrob pathGrob pointsGrob polylineGrob unit viewport nullGrob convertHeight convertWidth
+#' @importFrom methods as new slot slotNames
+#' @importFrom grid convertHeight convertUnit convertWidth current.viewport linesGrob nullGrob pathGrob pointsGrob polylineGrob unit viewport
 #' @import graphics
-#' @importFrom grDevices rgb dev.size
+#' @importFrom grDevices dev.size rgb
 #' @importFrom Rcpp evalCpp
-#' @importFrom DBI dbConnect dbDisconnect dbWriteTable dbGetQuery dbSendQuery dbReadTable dbExecute
-#' @importFrom units as_units set_units make_unit_label drop_units
+#' @importFrom DBI dbConnect dbDisconnect dbExecute dbGetQuery dbReadTable dbSendQuery dbWriteTable
+#' @importFrom units as_units drop_units make_unit_label set_units
 #' @importFrom classInt classIntervals
 #' @useDynLib sf
 NULL
@@ -25,6 +25,7 @@ setOldClass(c("sfc_POLYGON", "sfc"))
 setOldClass(c("sfc_MULTIPOLYGON", "sfc"))
 setOldClass(c("sfc_GEOMETRY", "sfc"))
 setOldClass("sfg")
+setOldClass("crs")
 
 .sf_cache <- new.env(FALSE, parent=globalenv())
 
@@ -34,8 +35,9 @@ pathGrob <- NULL
 		pathGrob <<- function(..., pathId.lengths) {
 			grid::pathGrob(...)
 		}
-	}
-	load_gdal() # nocov end
+	} # nocov end
+	load_gdal() 
+	assign(".sf.use_s2", Sys.getenv("_SF_USE_S2") == "true", envir=.sf_cache)
 }
 
 .onUnload = function(libname, pkgname) {
@@ -82,16 +84,6 @@ load_gdal <- function() {
 	}
 	CPL_gdal_init()
 	register_all_s3_methods() # dynamically registers non-imported pkgs (tidyverse)
-	if (inherits(try(units::as_units("link"), silent = TRUE), "try-error"))
-		units::install_conversion_constant("m", "link", 0.201168)
-	if (inherits(try(units::as_units("us_in"), silent = TRUE), "try-error"))
-		units::install_conversion_constant("m", "us_in", 1./39.37)
-	if (inherits(try(units::as_units("ind_yd"), silent = TRUE), "try-error"))
-		units::install_conversion_constant("m", "ind_yd", 0.91439523)
-	if (inherits(try(units::as_units("ind_ft"), silent = TRUE), "try-error"))
-		units::install_conversion_constant("m", "ind_ft", 0.30479841)
-	if (inherits(try(units::as_units("ind_ch"), silent = TRUE), "try-error"))
-		units::install_conversion_constant("m", "ind_ch", 20.11669506)
 }
 
 unload_gdal <- function() {
@@ -104,9 +96,24 @@ unload_gdal <- function() {
 		Sys.setenv("GDAL_DATA"=get(".sf.GDAL_DATA", envir=.sf_cache))
 		# nocov end
 	}
-	units::remove_symbolic_unit("link")
-	units::remove_symbolic_unit("us_in")
-	units::remove_symbolic_unit("ind_yd")
-	units::remove_symbolic_unit("ind_ft")
-	units::remove_symbolic_unit("ind_ch")
+	#units::remove_symbolic_unit("link")
+	#units::remove_symbolic_unit("us_in")
+	#units::remove_symbolic_unit("ind_yd")
+	#units::remove_symbolic_unit("ind_ft")
+	#units::remove_symbolic_unit("ind_ch")
+}
+ 
+
+#' @export
+#' @name sf_project
+#' @details \code{sf_add_proj_units} loads the PROJ units `link`, `us_in`, `ind_yd`, `ind_ft`, and `ind_ch` into the udunits database, and returns \code{TRUE} invisibly on success.
+sf_add_proj_units = function() {
+	#nocov start
+	units::install_conversion_constant("m", "link", 0.201168)
+	units::install_conversion_constant("m", "us_in", 1./39.37)
+	units::install_conversion_constant("m", "ind_yd", 0.91439523)
+	units::install_conversion_constant("m", "ind_ft", 0.30479841)
+	units::install_conversion_constant("m", "ind_ch", 20.11669506)
+	invisible(TRUE)
+	#nocov end
 }
