@@ -7,11 +7,11 @@
 #' @param y ignored
 #' @param ... further specifications, see \link{plot_sf} and \link{plot} and details.
 #' @param main title for plot (\code{NULL} to remove)
-#' @param pal palette function, similar to \link[grDevices]{rainbow}, or palette values; if omitted, \link{sf.colors} is used
+#' @param pal palette function, similar to \link{rainbow}, or palette values; if omitted, \code{sf.colors} is used
 #' @param nbreaks number of colors breaks (ignored for \code{factor} or \code{character} variables)
 #' @param breaks either a numeric vector with the actual breaks, or a name of a method accepted by the \code{style} argument of \link[classInt]{classIntervals}
 #' @param max.plot integer; lower boundary to maximum number of attributes to plot; the default value (9) can be overriden by setting the global option \code{sf_max.plot}, e.g. \code{options(sf_max.plot=2)}
-#' @param key.pos integer; side to plot a color key: 1 bottom, 2 left, 3 top, 4 right; set to \code{NULL} to omit key, or -1 to select automatically. If multiple columns are plotted in a single function call by default no key is plotted and every submap is stretched individually; if a key is requested (and \code{col} is missing) all maps are colored according to a single key. Auto select depends on plot size, map aspect, and, if set, parameter \code{asp}.
+#' @param key.pos integer; side to plot a color key: 1 bottom, 2 left, 3 top, 4 right; set to \code{NULL} to omit key completely, 0 to only not plot the key, or -1 to select automatically. If multiple columns are plotted in a single function call by default no key is plotted and every submap is stretched individually; if a key is requested (and \code{col} is missing) all maps are colored according to a single key. Auto select depends on plot size, map aspect, and, if set, parameter \code{asp}.
 #' @param key.width amount of space reserved for the key (incl. labels), thickness/width of the scale bar
 #' @param key.length amount of space reserved for the key along its axis, length of the scale bar
 #' @param pch plotting symbol
@@ -23,8 +23,11 @@
 #' @param border color of polygon border(s)
 #' @param add logical; add to current plot? Note that when using \code{add=TRUE}, you may have to set \code{reset=FALSE} in the first plot command.
 #' @param type plot type: 'p' for points, 'l' for lines, 'b' for both
-#' @param reset logical; if \code{FALSE}, keep the plot in a mode that allows adding further map elements; if \code{TRUE} restore original mode after plotting; see details.
+#' @param reset logical; if \code{FALSE}, keep the plot in a mode that allows adding further map elements; if \code{TRUE} restore original mode after plotting \code{sf} objects with attributes; see details.
 #' @param logz logical; if \code{TRUE}, use log10-scale for the attribute variable. In that case, \code{breaks} and \code{at} need to be given as log10-values; see examples.
+#' @param extent object with an \code{st_bbox} method to define plot extent; defaults to \code{x}
+#' @param xlim numeric; x-axis limits; overrides \code{extent}
+#' @param ylim numeric; y-axis limits; overrides \code{extent}
 #' @method plot sf
 #' @name plot
 #' @details \code{plot.sf} maximally plots \code{max.plot} maps with colors following from attribute columns,
@@ -36,7 +39,7 @@
 #' to control color, lines or symbols.
 #'
 #' When setting \code{reset} to \code{FALSE}, the original device parameters are lost, and the device must be reset using \code{dev.off()} in order to reset it.
-#' 
+#'
 #' parameter \code{at} can be set to specify where labels are placed along the key; see examples.
 #'
 #' @examples
@@ -62,8 +65,9 @@
 #' @export
 plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty",
 		max.plot = if(is.null(n <- options("sf_max.plot")[[1]])) 9 else n,
-		key.pos = get_key_pos(x, ...), key.length = .618, key.width = lcm(1.8), 
-		reset = TRUE, logz = FALSE) {
+		key.pos = get_key_pos(x, ...), key.length = .618, key.width = lcm(1.8),
+		reset = TRUE, logz = FALSE, extent = x, xlim = st_bbox(extent)[c(1,3)],
+		ylim = st_bbox(extent)[c(2,4)]) {
 
 	stopifnot(missing(y))
 	nbreaks.missing = missing(nbreaks)
@@ -72,6 +76,8 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 	dots = list(...)
 	col_missing = is.null(dots$col)
 
+	x = swap_axes_if_needed(x)
+
 	opar = par()
 	if (ncol(x) > 2 && !isTRUE(dots$add)) { # multiple maps to plot...
 		cols = setdiff(names(x), attr(x, "sf_column"))
@@ -105,7 +111,7 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 				v0 = values[!is.na(values)]
 				n.unq = length(unique(v0))
 				breaks = if (! all(is.na(values)) && n.unq > 1)
-						classInt::classIntervals(v0, min(nbreaks, n.unq), 
+						classInt::classIntervals(v0, min(nbreaks, n.unq),
 							breaks, warnSmallN = FALSE)$brks
 					else
 						range(values, na.rm = TRUE) # lowest and highest!
@@ -115,14 +121,14 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 
 		# loop over each map to plot:
 		lapply(cols, function(cname) plot(x[, cname], main = cname,
-			pal = pal, nbreaks = nbreaks, breaks = breaks, key.pos = NULL, reset = FALSE, 
-			logz = logz, ...))
+			pal = pal, nbreaks = nbreaks, breaks = breaks, key.pos = NULL, reset = FALSE,
+			logz = logz, xlim = xlim, ylim = ylim,...))
 
 		for (i in seq_len(prod(lt$mfrow) - length(cols))) # empty panels:
 			plot.new()
 
 		# plot key?
-		if (!is.null(key.pos) && col_missing) {
+		if (!is.null(key.pos) && key.pos != 0 && col_missing) {
 			if (is.null(pal))
 				pal = function(n) sf.colors(n, categorical = is.factor(values))
 			colors = if (is.function(pal))
@@ -133,15 +139,15 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 				.image_scale_factor(levels(values), colors, key.pos = key.pos,
 					key.width = key.width, key.length = key.length, ...)
 			else
-				.image_scale(values, colors, breaks = breaks, key.pos = key.pos, 
+				.image_scale(values, colors, breaks = breaks, key.pos = key.pos,
 					key.length = key.length, logz = logz, ...)
 		}
 
-	} else { # single map, or dots$add=TRUE:
-		if (!identical(TRUE, dots$add) && reset)
+	} else { # single map, or dots$add == TRUE:
+		if (!isTRUE(dots$add) && reset)
 			layout(matrix(1)) # reset
 		if (ncol(x) == 1) # no attributes to choose colors from: plot geometry
-			plot(st_geometry(x), ...)
+			plot(st_geometry(x), xlim = xlim, ylim = ylim, ...)
 		else { # generate plot with colors and possibly key
 			if (ncol(x) > 2) { # add = TRUE
 				warning("ignoring all but the first attribute")
@@ -226,7 +232,7 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 					.image_scale_factor(levels(values), colors, key.pos = key.pos,
 						key.width = key.width, key.length = key.length, ...)
 				} else
-					.image_scale(values, colors, breaks = breaks, key.pos = key.pos, 
+					.image_scale(values, colors, breaks = breaks, key.pos = key.pos,
 						key.length = key.length, logz = logz, ...)
 			}
 			# plot the map:
@@ -235,9 +241,9 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 				mar[1:2] = 2.1
 			par(mar = mar)
 			if (col_missing)
-				plot(st_geometry(x), col = col, ...)
+				plot(st_geometry(x), col = col, xlim = xlim, ylim = ylim, ...)
 			else
-				plot(st_geometry(x), ...)
+				plot(st_geometry(x), xlim = xlim, ylim = ylim, ...)
 		}
 		if (! isTRUE(dots$add)) { # title?
 			if (missing(main)) {
@@ -245,20 +251,29 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 				if (length(main) && inherits(x[[main]], "units"))
 					main = make_unit_label(main, x[[main]])
 			}
-			localTitle <- function(..., col, bg, pch, cex, lty, lwd, axes, type, bgMap, 
-					border, graticule, xlim, ylim, asp, bgc, xaxs, yaxs, lab, setParUsrBB, 
+			localTitle <- function(..., extent, col, bg, pch, cex, lty, lwd, axes, type, bgMap,
+					border, graticule, xlim, ylim, asp, bgc, xaxs, yaxs, lab, setParUsrBB,
 					expandBB, col_graticule, at, lon, lat, crs, datum, ndiscr, margin) # absorb
 				title(...)
 			localTitle(main, ...)
 		}
 	}
-	if (!isTRUE(dots$add) && reset) { # reset device:
+	if (!isTRUE(dots$add) && reset && ncol(x) > 1) { # reset device:
 		layout(matrix(1))
 		desel = which(names(opar) %in% c("cin", "cra", "csi", "cxy", "din", "page", "fig"))
 		par(opar[-desel])
 	}
 }
 
+swap_axes_if_needed = function(x) {
+	crs = st_crs(x)
+	if (st_axis_order() && !is.na(crs) && crs_parameters(crs)$yx)
+		st_transform(x, pipeline = "+proj=pipeline +step +proj=axisswap +order=2,1")
+	else
+		x
+}
+
+
 #' @name plot
 #' @export
 get_key_pos = function(x, ...) {
@@ -540,6 +555,7 @@ plot_sf = function(x, xlim = NULL, ylim = NULL, asp = NA, axes = FALSE, bgc = pa
 #   min max
 # x
 # y
+
 	bbox = matrix(st_bbox(x), 2, dimnames = list(c("x", "y"), c("min", "max")))
 	# expandBB: 1=below, 2=left, 3=above and 4=right.
 	expBB = function(lim, expand) c(lim[1] - expand[1] * diff(lim), lim[2] + expand[2] * diff(lim))
@@ -621,7 +637,7 @@ plot_sf = function(x, xlim = NULL, ylim = NULL, asp = NA, axes = FALSE, bgc = pa
 #' @name plot
 #' @export
 #' @details non-categorical colors from \code{sf.colors} were taken from \link[sp]{bpy.colors}, with modified \code{cutoff.tails} defaults
-#' If categorical is \code{TRUE}, default colors are from \url{http://www.colorbrewer2.org/} (if n < 9, Set2, else Set3).
+#' If categorical is \code{TRUE}, default colors are from \url{https://colorbrewer2.org/} (if n < 9, Set2, else Set3).
 #' @examples
 #' sf.colors(10)
 sf.colors = function (n = 10, cutoff.tails = c(0.35, 0.2), alpha = 1, categorical = FALSE) {
@@ -651,25 +667,32 @@ sf.colors = function (n = 10, cutoff.tails = c(0.35, 0.2), alpha = 1, categorica
 #' @param n ignore
 #' @param total_size ignore
 #' @param key.length ignore
-.get_layout = function(bb, n, total_size, key.pos, key.length) {
+#' @param mfrow length-2 integer vector with number of rows, columns
+.get_layout = function(bb, n, total_size, key.pos, key.length, mfrow = NULL) {
 # return list with "m" matrix, "key.pos", "widths" and "heights" fields
 # if key.pos = -1, it will be a return value, "optimally" placed
 	asp = diff(bb[c(2,4)])/diff(bb[c(1,3)])
 	if (isTRUE(st_is_longlat(bb)))
 		asp = asp / cos(mean(bb[c(2,4)]) * pi /180)
-	size = function(nrow, n, asp) {
-		ncol = ceiling(n / nrow)
-		xsize = total_size[1] / ncol
-		ysize = xsize  * asp
-		if (xsize * ysize * n > prod(total_size)) {
-			ysize = total_size[2] / nrow
-			xsize = ysize / asp
+	if (is.null(mfrow)) {
+		size = function(nrow, n, asp) {
+			ncol = ceiling(n / nrow)
+			xsize = total_size[1] / ncol
+			ysize = xsize  * asp
+			if (xsize * ysize * n > prod(total_size)) {
+				ysize = total_size[2] / nrow
+				xsize = ysize / asp
+			}
+			xsize * ysize
 		}
-		xsize * ysize
+		sz = vapply(1:n, function(x) size(x, n, asp), 0.0)
+		nrow = which.max(sz)
+		ncol = ceiling(n / nrow)
+	} else {
+		stopifnot(is.numeric(mfrow), length(mfrow) == 2)
+		nrow = mfrow[1]
+		ncol = mfrow[2]
 	}
-	sz = vapply(1:n, function(x) size(x, n, asp), 0.0)
-	nrow = which.max(sz)
-	ncol = ceiling(n / nrow)
 
 	ret = list()
 	ret$mfrow = c(nrow, ncol)
@@ -792,7 +815,7 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 		c(0,0,-offset,-offset),
 		c(0,0,-offset,-offset),
 		c(offset,offset,0,0),
-		c(offset,offset,0,0)) 
+		c(offset,offset,0,0))
 	for(i in seq_along(poly)) {
 		if (key.pos %in% c(1,3))
 			polygon(poly[[i]], c(0, 0, 1, 1) + offs, col = col[i], border = NA)
@@ -821,7 +844,7 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 #' @name stars
 #' @export
 #' @param key.width ignore
-.image_scale_factor = function(z, col, breaks = NULL, key.pos, add.axis = TRUE,
+.image_scale_factor = function(z, col, key.pos, add.axis = TRUE,
 	..., axes = FALSE, key.width, key.length) {
 
 	n = length(z)
