@@ -1,3 +1,10 @@
+sf_stringsAsFactors = function() {
+	if (getRversion() < "4.1.0")
+		default.stringsAsFactors()
+	else
+		FALSE
+}
+
 set_utf8 = function(x) {
 	n = names(x)
 	Encoding(n) = "UTF-8"
@@ -28,7 +35,9 @@ set_utf8 = function(x) {
 #'   use the \code{query} argument.
 #' @param ... parameter(s) passed on to \link{st_as_sf}
 #' @param options character; driver dependent dataset open options, multiple
-#'   options supported.
+#'   options supported. For possible values, see the "Open options" section
+#'   of the GDAL documentation of the corresponding driver, and
+#'   https://github.com/r-spatial/sf/issues/1157 for an example.
 #' @param quiet logical; suppress info on name, driver, size and spatial
 #'   reference, or signaling no or multiple layers
 #' @param geometry_column integer or character; in case of multiple geometry
@@ -42,8 +51,9 @@ set_utf8 = function(x) {
 #'   of LineString and MultiLineString, or of Polygon and MultiPolygon, convert
 #'   all to the Multi variety; defaults to \code{TRUE}
 #' @param stringsAsFactors logical; logical: should character vectors be
-#'   converted to factors?  The `factory-fresh' default is \code{TRUE}, but this
-#'   can be changed by setting \code{options(stringsAsFactors = FALSE)}.
+#'   converted to factors?  Default for \code{read_sf} or R version >= 4.1.0 is 
+#' \code{FALSE}, for \code{st_read} and R version < 4.1.0 equal to
+#' \code{default.stringsAsFactors()}
 #' @param int64_as_string logical; if TRUE, Int64 attributes are returned as
 #'   string; if FALSE, they are returned as double and a warning is given when
 #'   precision is lost (i.e., values are larger than 2^53).
@@ -108,6 +118,10 @@ set_utf8 = function(x) {
 #' nc_gpkg_sql = st_read(system.file("gpkg/nc.gpkg", package = "sf"),
 #'    query = sprintf("SELECT NAME, SID74, FIPS, geom  FROM \"%s\" WHERE BIR74 > 20000", layer))
 #' }
+#' # spatial filter, as wkt:
+#' wkt = st_as_text(st_geometry(nc[1,]))
+#' # filter by (bbox overlaps of) first feature geometry:
+#' read_sf(system.file("gpkg/nc.gpkg", package="sf"), wkt_filter = wkt)
 #' @export
 st_read = function(dsn, layer, ...) UseMethod("st_read")
 
@@ -127,7 +141,8 @@ st_read.default = function(dsn, layer, ...) {
 }
 
 process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
-		stringsAsFactors = ifelse(as_tibble, FALSE, default.stringsAsFactors()), geometry_column = 1, as_tibble = FALSE) {
+		stringsAsFactors = ifelse(as_tibble, FALSE, sf_stringsAsFactors()), 
+		geometry_column = 1, as_tibble = FALSE) {
 
 	which.geom = which(vapply(x, function(f) inherits(f, "sfc"), TRUE))
 
@@ -136,8 +151,8 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 
 	# in case no geometry is present:
 	if (length(which.geom) == 0) {
-		warning("no simple feature geometries present: returning a data.frame or tbl_df",
-			call. = FALSE)
+		if (! quiet) 
+			warning("no simple feature geometries present: returning a data.frame or tbl_df", call. = FALSE)
 		x = if (!as_tibble) {
 				if (any(sapply(x, is.list)))
 					warning("list-column(s) present: in case of failure, try read_sf or as_tibble=TRUE") # nocov
@@ -164,7 +179,7 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 		x <- as.data.frame(set_utf8(x[-c(lc.other, which.geom)]), stringsAsFactors = stringsAsFactors)
 		if (as_tibble) {
 			# "sf" class is added later by `st_as_sf` (and sets all the attributes)
-			x <- tibble::new_tibble(x, attributes(x), nrow = nrow(x))
+			x <- tibble::new_tibble(x, nrow = nrow(x))
 		}
 	}
 
@@ -186,15 +201,16 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 #' @name st_read
 #' @param fid_column_name character; name of column to write feature IDs to; defaults to not doing this
 #' @param drivers character; limited set of driver short names to be tried (default: try all)
+#' @param wkt_filter character; WKT representation of a spatial filter (may be used as bounding box, selecting overlapping geometries); see examples
 #' @note The use of \code{system.file} in examples make sure that examples run regardless where R is installed:
 #' typical users will not use \code{system.file} but give the file name directly, either with full path or relative
 #' to the current working directory (see \link{getwd}). "Shapefiles" consist of several files with the same basename
 #' that reside in the same directory, only one of them having extension \code{.shp}.
 #' @export
-st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet = FALSE, geometry_column = 1L, type = 0,
-		promote_to_multi = TRUE, stringsAsFactors = default.stringsAsFactors(),
+st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet = FALSE, geometry_column = 1L, 
+		type = 0, promote_to_multi = TRUE, stringsAsFactors = sf_stringsAsFactors(),
 		int64_as_string = FALSE, check_ring_dir = FALSE, fid_column_name = character(0),
-		drivers = character(0)) {
+		drivers = character(0), wkt_filter = character(0)) {
 
 	layer = if (missing(layer))
 		character(0)
@@ -212,7 +228,7 @@ st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet
 		stop("`promote_to_multi' should have length one, and applies to all geometry columns")
 
 	x = CPL_read_ogr(dsn, layer, query, as.character(options), quiet, type, fid_column_name,
-		drivers, promote_to_multi, int64_as_string, dsn_exists, dsn_isdb)
+		drivers, wkt_filter, promote_to_multi, int64_as_string, dsn_exists, dsn_isdb)
 	process_cpl_read_ogr(x, quiet, check_ring_dir = check_ring_dir,
 		stringsAsFactors = stringsAsFactors, geometry_column = geometry_column, ...)
 }
@@ -255,6 +271,10 @@ clean_columns = function(obj, factorsAsCharacter) {
 	ccls.ok = vapply(obj, function(x) inherits(x, permitted), TRUE)
 	if (any(!ccls.ok)) {
 		# nocov start
+                nms <- names(obj)[!ccls.ok]
+                cls <- sapply(obj, function(x) paste(class(x), collapse=";"))[!ccls.ok]
+                warning("Dropping column(s) ", paste(nms, collapse=","),
+                    " of class(es) ", paste(cls, collapse=","))
 		obj = obj[ccls.ok]
 		# nocov end
 	}
@@ -296,41 +316,38 @@ abbreviate_shapefile_names = function(x) {
 #' \link{basename} of \code{dsn} is taken.
 #' @param driver character; name of driver to be used; if missing and \code{dsn} is not a Database Connection, a driver name is guessed from \code{dsn};
 #' \code{st_drivers()} returns the drivers that are available with their properties; links to full driver documentation
-#' are found at \url{http://www.gdal.org/ogr_formats.html}.
+#' are found at \url{https://gdal.org/ogr_formats.html}.
 #' @param ... other arguments passed to \link{dbWriteTable} when \code{dsn} is a
 #' Database Connection
-#' @param dataset_options character; driver dependent dataset creation options; 
+#' @param dataset_options character; driver dependent dataset creation options;
 #' multiple options supported.
-#' @param layer_options character; driver dependent layer creation options; 
+#' @param layer_options character; driver dependent layer creation options;
 #' multiple options supported.
 #' @param quiet logical; suppress info on name, driver, size and spatial reference
-#' @param factorsAsCharacter logical; convert \code{factor} objects into 
+#' @param factorsAsCharacter logical; convert \code{factor} objects into
 #' character strings (default), else into numbers by \code{as.numeric}.
-#' @param update logical; \code{FALSE} by default for single-layer drivers but 
-#' \code{TRUE} by default for database drivers as defined by \code{db_drivers}.
-#' For database-type drivers (e.g. GPKG) \code{TRUE} values will make 
-#' \code{GDAL} try to update (append to) the existing data source, e.g. adding
-#' a table to an existing database, or adding records to a layer. See also the
-#' next two arguments and Details.
-#' @param delete_dsn logical; delete data source \code{dsn} before attempting 
+#' @param append logical; should we append to an existing layer, or replace it?
+#' if \code{TRUE} append, if \code{FALSE} replace; default \code{NA} will
+#' raise an error if the layer exists. See also next two arguments.
+#' @param delete_dsn logical; delete data source \code{dsn} before attempting
 #' to write?
 #' @param delete_layer logical; delete layer \code{layer} before attempting to
 #' write?
 #' @param fid_column_name character, name of column with feature IDs; if
 #' specified, this column is no longer written as feature attribute.
-#' @details 
-#' Columns (variables) of a class not supported are dropped with a warning. 
-#' 
+#' @details
+#' Columns (variables) of a class not supported are dropped with a warning.
+#'
 #' When updating an existing layer, records are appended to it if the updating
-#' object has the right variable names and types. If names don't match an 
+#' object has the right variable names and types. If names don't match an
 #' error is raised. If types don't match, behaviour is undefined: GDAL may
 #' raise warnings or errors or fail silently.
-#' 
-#' When deleting layers or data sources is not successful, no error is emitted. 
-#' \code{delete_dsn} and \code{delete_layers} should be
+#'
+#' When deleting layers or data sources is not successful, no error is emitted.
+#' \code{delete_dsn} and \code{delete_layer} should be
 #' handled with care; the former may erase complete directories or databases.
 #' @seealso \link{st_drivers}
-#' @return \code{obj}, invisibly; in case \code{obj} is of class \code{sfc}, 
+#' @return \code{obj}, invisibly; in case \code{obj} is of class \code{sfc},
 #' it is returned as an \code{sf} object.
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
@@ -339,7 +356,7 @@ abbreviate_shapefile_names = function(x) {
 #' data(meuse, package = "sp") # loads data.frame from sp
 #' meuse_sf = st_as_sf(meuse, coords = c("x", "y"), crs = 28992)
 #' # writes X and Y as columns:
-#' st_write(meuse_sf, paste0(tempdir(), "/", "meuse.csv"), layer_options = "GEOMETRY=AS_XY") 
+#' st_write(meuse_sf, paste0(tempdir(), "/", "meuse.csv"), layer_options = "GEOMETRY=AS_XY")
 #' st_write(meuse_sf, paste0(tempdir(), "/", "meuse.csv"), layer_options = "GEOMETRY=AS_WKT",
 #'   delete_dsn=TRUE) # overwrites
 #' \dontrun{
@@ -367,9 +384,16 @@ st_write.sfc = function(obj, dsn, layer, ...) {
 st_write.sf = function(obj, dsn, layer = NULL, ...,
 		driver = guess_driver_can_write(dsn),
 		dataset_options = NULL, layer_options = NULL, quiet = FALSE, factorsAsCharacter = TRUE,
-		update = driver %in% db_drivers, delete_dsn = FALSE, delete_layer = FALSE,
+		append = NA, delete_dsn = FALSE, delete_layer = !is.na(append) && !append,
 		fid_column_name = NULL) {
 
+	if (!is.null(list(...)$update)) {
+		.Deprecated("append", old = "update") # deprecated at 0.9-0
+		if (is.na(append))
+			append = list(...)$update
+	}
+#	else if (length(list(...)))
+#		stop(paste("unrecognized argument(s)", names(list(...)), "\n"))
 	if (missing(dsn))
 		stop("dsn should specify a data source or filename")
 	if (inherits(dsn, c("DBIObject", "PostgreSQLConnection", "Pool"))) {
@@ -379,17 +403,26 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 			dsn = pool::poolCheckout(dsn)
 			on.exit(pool::poolReturn(dsn)) # nocov end
 		}
-		if (is.null(layer))
+
+		if (is.null(layer)) {
 			layer = deparse(substitute(obj))
-		dbWriteTable(dsn, name = layer, value = obj, ...,
-			factorsAsCharacter = factorsAsCharacter)
+		}
+
+		if (is.na(append)) {
+			append = FALSE
+		}
+
+		dbWriteTable(dsn, name = layer, value = obj,
+					 append = append, overwrite = delete_layer,
+					 factorsAsCharacter = factorsAsCharacter, ...)
 		return(invisible(obj))
 	} else if (!inherits(dsn, "character")) { # add methods for other dsn classes here...
 		stop(paste("no st_write method available for dsn of class", class(dsn)[1]))
 	}
 
-	if (length(list(...)))
-		stop(paste("unrecognized argument(s)", names(list(...)), "\n"))
+	if (!is.na(append) && append == FALSE && delete_layer == FALSE)
+		stop("cannot replace a layer if delete_layer is FALSE")
+
 	if (is.null(layer))
 		layer <- file_path_sans_ext(basename(dsn))
 
@@ -399,8 +432,15 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 	# this seems to be always a good idea:
 	dsn = enc2utf8(dsn)
 
-	geom = st_geometry(obj)
-	obj[[attr(obj, "sf_column")]] = NULL
+	# handle the case where obj does not have a geometry column:
+	if (write_geometries <- inherits(obj, "sf")) {
+		geom = st_geometry(obj)
+		obj[[attr(obj, "sf_column")]] = NULL
+	} else { # create fake geometries:
+		v = vector("list", nrow(obj))
+		v[seq_len(nrow(obj))] = list(st_point())
+		geom = st_sfc(v)
+	}
 
 	if (driver == "ESRI Shapefile") { # remove trailing .shp from layer name
 		layer = sub(".shp$", "", layer)
@@ -426,16 +466,18 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 
 	ret = CPL_write_ogr(obj, dsn, layer, driver,
 		as.character(dataset_options), as.character(layer_options),
-		geom, dim, fids, quiet, update, delete_dsn, delete_layer)
+		geom, dim, fids, quiet, append, delete_dsn, delete_layer,
+		write_geometries)
 	if (ret == 1) { # try through temp file:
 		tmp = tempfile(fileext = paste0(".", tools::file_ext(dsn))) # nocov start
 		if (!quiet)
 			message(paste("writing first to temporary file", tmp))
 		if (CPL_write_ogr(obj, tmp, layer, driver,
 				as.character(dataset_options), as.character(layer_options),
-				geom, dim, fids, quiet, update, delete_dsn, delete_layer) == 1)
+				geom, dim, fids, quiet, append, delete_dsn, delete_layer,
+				write_geometries) == 1)
 			stop(paste("failed writing to temporary file", tmp))
-		if (!file.copy(tmp, dsn, overwrite = update || delete_dsn || delete_layer))
+		if (!file.copy(tmp, dsn, overwrite = append || delete_dsn || delete_layer))
 			stop(paste("copying", tmp, "to", dsn, "failed"))
 		if (!file.remove(tmp))
 			warning(paste("removing", tmp, "failed"))
@@ -446,13 +488,17 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 #' @name st_write
 #' @export
 st_write.data.frame <- function(obj, dsn, layer = NULL, ...) {
-    st_write.sf(obj = st_as_sf(obj), dsn = dsn, layer = layer, ...)
+    sf = try(st_as_sf(obj), silent = TRUE)
+	if (!inherits(sf, "try-error"))
+    	st_write.sf(sf, dsn = dsn, layer = layer, ...)
+	else
+    	st_write.sf(obj, dsn = dsn, layer = layer, ...)
 }
 
 #' @name st_write
 #' @export
-write_sf <- function(..., quiet = TRUE, delete_layer = TRUE) {
-	st_write(..., quiet = quiet, delete_layer = delete_layer)
+write_sf <- function(..., quiet = TRUE, append = FALSE, delete_layer = TRUE) {
+	st_write(..., quiet = quiet, append = append, delete_layer = TRUE)
 }
 
 #' Get GDAL drivers
@@ -597,6 +643,7 @@ extension_map <- list(
         "bna" = "BNA",
         "csv" = "CSV",
         "e00" = "AVCE00",
+        "fgb" = "FlatGeobuf",
         "gdb" = "OpenFileGDB",
         "geojson" = "GeoJSON",
         "gml" = "GML",
