@@ -49,10 +49,10 @@ st_as_sf.data.frame = function(x, ..., agr = NA_agr_, coords, wkt,
 		x$geometry = structure( points_rcpp(as.matrix(cc), dim),
 			n_empty = 0L, precision = 0, crs = NA_crs_,
 			bbox = structure(
-				c(xmin = min(x[[coords[1]]], na.rm = TRUE),
-				ymin = min(x[[coords[2]]], na.rm = TRUE),
-				xmax = max(x[[coords[1]]], na.rm = TRUE),
-				ymax = max(x[[coords[2]]], na.rm = TRUE)), class = "bbox"),
+				c(xmin = min(cc[[1]], na.rm = TRUE),
+				ymin = min(cc[[2]], na.rm = TRUE),
+				xmax = max(cc[[1]], na.rm = TRUE),
+				ymax = max(cc[[2]], na.rm = TRUE)), class = "bbox"),
 			class =  c("sfc_POINT", "sfc" ), names = NULL)
 
 		if (is.character(coords))
@@ -190,10 +190,10 @@ list_column_to_sfc = function(x) {
 #' Create sf, which extends data.frame-like objects with a simple feature list column
 #' @name sf
 #' @param ... column elements to be binded into an \code{sf} object or a single \code{list} or \code{data.frame} with such columns; at least one of these columns shall be a geometry list-column of class \code{sfc} or be a list-column that can be converted into an \code{sfc} by \link{st_as_sfc}.
-#' @param crs coordinate reference system: integer with the EPSG code, or character with proj4string
+#' @param crs coordinate reference system, something suitable as input to \link{st_crs}
 #' @param agr character vector; see details below.
 #' @param row.names row.names for the created \code{sf} object
-#' @param stringsAsFactors logical; logical: should character vectors be converted to factors?  The `factory-fresh' default is \code{TRUE}, but this can be changed by setting \code{options(stringsAsFactors = FALSE)}.
+#' @param stringsAsFactors logical; see \link{st_read}
 #' @param precision numeric; see \link{st_as_binary}
 #' @param sf_column_name character; name of the active list-column with simple feature geometries; in case
 #' there is more than one and \code{sf_column_name} is \code{NULL}, the first one is taken.
@@ -201,7 +201,9 @@ list_column_to_sfc = function(x) {
 #' @param check_ring_dir see \link{st_read}
 #' @details \code{agr}, attribute-geometry-relationship, specifies for each non-geometry attribute column how it relates to the geometry, and can have one of following values: "constant", "aggregate", "identity". "constant" is used for attributes that are constant throughout the geometry (e.g. land use), "aggregate" where the attribute is an aggregate value over the geometry (e.g. population density or population count), "identity" when the attributes uniquely identifies the geometry of particular "thing", such as a building ID or a city name. The default value, \code{NA_agr_}, implies we don't know.
 #'
-#' When confronted with a data.frame-like object, `st_sf` will try to find a geometry column of class `sfc`, and otherwise try to convert list-columns when available into a geometry column, using \link{st_as_sfc}.
+#' When a single value is provided to \code{agr}, it is cascaded across all input columns; otherwise, a named vector like \code{c(feature1='constant', ...)} will set \code{agr} value to \code{'constant'} for the input column named \code{feature1}. See \code{demo(nc)} for a worked example of this.
+#'
+#' When confronted with a data.frame-like object, \code{st_sf} will try to find a geometry column of class \code{sfc}, and otherwise try to convert list-columns when available into a geometry column, using \link{st_as_sfc}.
 #' @examples
 #' g = st_sfc(st_point(1:2))
 #' st_sf(a=3,g)
@@ -213,7 +215,7 @@ list_column_to_sfc = function(x) {
 #' df <- st_sf(id = 1:nrows, geometry = geometry)
 #' @export
 st_sf = function(..., agr = NA_agr_, row.names,
-		stringsAsFactors = default.stringsAsFactors(), crs, precision, 
+		stringsAsFactors = sf_stringsAsFactors(), crs, precision,
 		sf_column_name = NULL, check_ring_dir = FALSE, sfc_last = TRUE) {
 	x = list(...)
 	if (length(x) == 1L && (inherits(x[[1L]], "data.frame") || (is.list(x) && !inherits(x[[1L]], "sfc"))))
@@ -263,13 +265,13 @@ st_sf = function(..., agr = NA_agr_, row.names,
 		else if (sfc_last && inherits(x, "data.frame"))
 			x[-all_sfc_columns]
 		else
-			cbind(data.frame(row.names = row.names), 
-				as.data.frame(x[-all_sfc_columns], 
+			cbind(data.frame(row.names = row.names),
+				as.data.frame(x[-all_sfc_columns],
 					stringsAsFactors = stringsAsFactors, optional = TRUE))
 
 	if (check_ring_dir) { # process:
 		for (i in seq_along(all_sfc_names))
-			df[[ all_sfc_names[i] ]] = st_sfc(x[[ all_sfc_columns[i] ]], 
+			df[[ all_sfc_names[i] ]] = st_sfc(x[[ all_sfc_columns[i] ]],
 				check_ring_dir = check_ring_dir)
 	} else { # copy:
 		for (i in seq_along(all_sfc_names))
@@ -379,7 +381,7 @@ st_sf = function(..., agr = NA_agr_, row.names,
 				agr[ix] = NA
 		}
 	}
-	x = NextMethod()
+	x = structure(NextMethod(), class = c("sf", setdiff(class(x), "sf")))
 	if (! setting_geom)
 		st_agr(x) = agr
 	x
@@ -430,11 +432,11 @@ merge.sf = function(x, y, ...) {
 	if (inherits(y, "sf"))
 		stop("merge on two sf objects not supported")
 	sf_column = attr(x, "sf_column")
-	ret = NextMethod()
+	ret = NextMethod() # if data.table, drops sf_column attribute;
+	class(ret) = setdiff(class(ret), "sf")
 	g = ret[[sf_column]] # may have NULL values in it
 	ret[[sf_column]] = NULL
-	st_geometry(ret) = st_sfc(g)
-	ret
+	st_set_geometry(ret, st_sfc(g)) # FIXME: set agr
 }
 
 #' @export
@@ -451,3 +453,21 @@ st_drop_geometry = function(x) {
 		stop("st_drop_geometry only works with objects of class sf")
 	st_set_geometry(x, NULL)
 }
+
+#' transform method for sf objects
+#' 
+#' Can be used to create or modify attribute variables; for transforming geometries see 
+#' \link{st_transform}, and all other functions starting with \code{st_}.
+#' 
+#' @param _data object of class \code{sf}
+#' @param ... Further arguments of the form new_variable=expression
+#'
+#' @export
+#' @examples
+#' a = data.frame(x1 = 1:3, x2 = 5:7)
+#' st_geometry(a) = st_sfc(st_point(c(0,0)), st_point(c(1,1)), st_point(c(2,2)))
+#' transform(a, x1_sq = x1^2)
+#' transform(a, x1_x2 = x1*x2)
+transform.sf <- function (`_data`, ...) {
+  st_as_sf(NextMethod(), agr = st_agr(`_data`), sf_column_name = attr(`_data`, "sf_column"))
+}
