@@ -18,10 +18,12 @@ format.sfc = function(x, ..., width = 30) {
 #' Create simple feature geometry list column, set class, and add coordinate reference system and precision
 #'
 #' @name sfc
+#' @aliases sfc_POINT sfc_LINESTRING sfc_POLYGON sfc_MULTIPOINT sfc_MULTILINESTRING sfc_MULTIPOLYGON sfc_GEOMETRYCOLLECTION
 #' @param ... zero or more simple feature geometries (objects of class \code{sfg}), or a single list of such objects; \code{NULL} values will get replaced by empty geometries.
 #' @param crs coordinate reference system: integer with the EPSG code, or character with proj4string
 #' @param precision numeric; see \link{st_as_binary}
 #' @param check_ring_dir see \link{st_read}
+#' @param dim character; if this function is called without valid geometries, this argument may carry the right dimension to set empty geometries
 #' @return an object of class \code{sfc}, which is a classed list-column with simple feature geometries.
 #'
 #' @details A simple feature geometry list-column is a list of class
@@ -34,7 +36,7 @@ format.sfc = function(x, ..., width = 30) {
 #' (sfc = st_sfc(pt1, pt2))
 #' d = st_sf(data.frame(a=1:2, geom=sfc))
 #' @export
-st_sfc = function(..., crs = NA_crs_, precision = 0.0, check_ring_dir = FALSE) {
+st_sfc = function(..., crs = NA_crs_, precision = 0.0, check_ring_dir = FALSE, dim) {
 	lst = list(...)
 	# if we have only one arg, which is already a list with sfg's, but NOT a geometrycollection:
 	# (this is the old form of calling st_sfc; it is way faster to call st_sfc(lst) if lst
@@ -68,11 +70,18 @@ st_sfc = function(..., crs = NA_crs_, precision = 0.0, check_ring_dir = FALSE) {
 	}
 
 	if (any(is_null)) {
+		if (missing(dim)) {
+			dim = if (length(lst) == 0) # we have no clue:
+					"XY"
+				else
+					sfg_classes[1L, 1L]
+		}
 		ret = vector("list", length(is_null))
 		ret[!is_null] = lst
-		ret[ is_null] = list(typed_empty(cls))
+		ret[ is_null] = list(typed_empty(cls, nchar(dim), dim = dim))
 		attributes(ret) = attributes(lst)
 		lst = ret
+		sfg_classes = vapply(lst, class, rep(NA_character_, 3))
 	}
 
 	# set class:
@@ -87,6 +96,20 @@ st_sfc = function(..., crs = NA_crs_, precision = 0.0, check_ring_dir = FALSE) {
 	if (is.null(bb) || any(is.na(bb)))
 		attr(lst, "bbox") = compute_bbox(lst)
 
+	# compute z_range, if dims permit and not set
+	zr = attr(lst, "z_range")
+	if (is.null(zr) || any(is.na(zr))) {
+		u <- unique(sfg_classes[1L,])
+		if( "XYZM" %in% u ) {
+			attr(lst, "z_range") = compute_z_range(lst)
+			attr(lst, "m_range") = compute_m_range(lst)
+		} else if ( "XYZ" %in% u ) {
+			attr(lst, "z_range") = compute_z_range(lst)
+		} else if ("XYM" %in% u ) {
+			attr(lst, "m_range") = compute_m_range(lst)
+		}
+	}
+
 	# check ring directions:
 	if (check_ring_dir) # also GEOMETRYCOLLECTION?
 		lst = check_ring_dir(lst)
@@ -121,7 +144,8 @@ sfg_is_empty = function(x) {
 "[.sfc" = function(x, i, j, ..., op = st_intersects) {
 	if (!missing(i) && (inherits(i, "sf") || inherits(i, "sfc") || inherits(i, "sfg")))
 		i = lengths(op(x, i, ...)) != 0
-	st_sfc(NextMethod(), crs = st_crs(x), precision = st_precision(x))
+	st_sfc(unclass(x)[i], crs = st_crs(x), precision = st_precision(x), 
+		dim = if(length(x)) class(x[[1]])[1] else "XY")
 }
 
 
@@ -177,9 +201,36 @@ print.sfc = function(x, ..., n = 5L, what = "Geometry set for", append = "") {
 	bb = signif(attr(x, "bbox"), options("digits")$digits)
 	cat(paste(paste(names(bb), bb[], sep = ": "), collapse = " "))
 	cat("\n")
+	if( !is.null( attr(x, "z_range"))) {
+		cat(paste0("z_range:        "))
+		zb = signif(attr(x, "z_range"), options("digits")$digits)
+		cat(paste(paste(names(zb), zb[], sep = ": "), collapse = " "))
+		cat("\n")
+	}
+	if( !is.null( attr(x, "m_range"))) {
+		cat(paste0("m_range:        "))
+		mb = signif(attr(x, "m_range"), options("digits")$digits)
+		cat(paste(paste(names(mb), mb[], sep = ": "), collapse = " "))
+		cat("\n")
+	}
 	# attributes: epsg, proj4string, precision
-	cat(paste0("epsg (SRID):    ", attr(x, "crs")$epsg, "\n"))
-	cat(paste0("proj4string:    ", attr(x, "crs")$proj4string, "\n"))
+	crs = st_crs(x)
+	if (is.na(crs))
+		cat(paste0("CRS:            NA\n"))
+	else {
+		p = crs_parameters(crs)
+		if (p$Name == "unknown") {
+			if (!is.character(crs$input) || is.na(crs$input))
+				cat(paste0("proj4string:    ", crs$proj4string, "\n"))
+			else
+				cat(paste0("CRS:            ", crs$input, "\n"))
+		} else if (p$IsGeographic)
+			cat(paste0("geographic CRS: ", p$Name, "\n"))
+		else
+			cat(paste0("projected CRS:  ", p$Name, "\n"))
+#		if (!is.na(crs$epsg))
+#			cat(paste0("epsg (SRID):    ", crs$epsg, "\n"))
+	}
 	if (attr(x, "precision") != 0.0) {
 		cat(paste0("precision:      "))
 		if (attr(x, "precision") < 0.0)
@@ -208,7 +259,7 @@ print.sfc = function(x, ..., n = 5L, what = "Geometry set for", append = "") {
 #' @export
 summary.sfc = function(object, ..., maxsum = 7L, maxp4s = 10L) {
 	u = factor(vapply(object, function(x) WKT_name(x, FALSE), ""))
-    epsg = paste0("epsg:", attr(object, "crs")$epsg)
+    epsg = paste0("epsg:", st_crs(object)$epsg)
 	levels(u) = c(levels(u), epsg)
     p4s = attr(object, "crs")$proj4string
 	if (!is.na(p4s)) {
@@ -235,11 +286,18 @@ st_geometry.sfc = function(obj, ...) obj
 #'
 #' Return geometry type of an object, as a factor
 #' @param x object of class \link{sf} or \link{sfc}
-#' @return a factor with the geometry type of each simple feature in x
+#' @param by_geometry logical; if \code{TRUE}, return geometry type of each geometry,
+#' else return geometry type of the set
+#' @return a factor with the geometry type of each simple feature geometry
+#' in \code{x}, or that of the whole set
 #' @export
-st_geometry_type = function(x) {
+st_geometry_type = function(x, by_geometry = TRUE) {
 	x = st_geometry(x)
-	factor(vapply(x, function(y) class(y)[2], ""), levels =
+	f = if (by_geometry)
+			vapply(x, function(y) class(y)[2], "")
+		else
+			substring(class(x)[1], 5)
+	factor(f, levels =
 		c("GEOMETRY",
 		"POINT",
 		"LINESTRING",
@@ -350,11 +408,11 @@ st_precision.sfc <- function(x) {
 #' @name st_precision
 #' @param precision numeric, or object of class \code{units} with distance units (but see details); see \link{st_as_binary} for how to do this.
 #' @details If \code{precision} is a \code{units} object, the object on which we set precision must have a coordinate reference system with compatible distance units.
-#' 
+#'
 #' Setting a \code{precision} has no direct effect on coordinates of geometries, but merely set an attribute tag to an \code{sfc} object. The effect takes place in \link{st_as_binary} or, more precise, in the C++ function \code{CPL_write_wkb}, where simple feature geometries are being serialized to well-known-binary (WKB). This happens always when routines are called in GEOS library (geometrical operations or predicates), for writing geometries using \link{st_write} or \link{write_sf}, \code{st_make_valid} in package \code{lwgeom}; also \link{aggregate} and \link{summarise} by default union geometries, which calls a GEOS library function. Routines in these libraries receive rounded coordinates, and possibly return results based on them. \link{st_as_binary} contains an example of a roundtrip of \code{sfc} geometries through WKB, in order to see the rounding happening to R data.
 #'
 #' The reason to support precision is that geometrical operations in GEOS or liblwgeom may work better at reduced precision. For writing data from R to external resources it is harder to think of a good reason to limiting precision.
-#' 
+#'
 #' @seealso \link{st_as_binary} for an explanation of what setting precision does, and the examples therein.
 #' @examples
 #' x <- st_sfc(st_point(c(pi, pi)))
@@ -399,15 +457,15 @@ st_set_precision.sf <- function(x, precision) {
     st_set_precision(x, value)
 }
 
-typed_empty = function(cls) {
+typed_empty = function(cls, ncol = 2, dim = "XY") {
 	switch(cls[1],
-		sfc_POINT = st_point(),
-		sfc_MULTIPOINT = st_multipoint(),
-		sfc_LINESTRING = st_linestring(),
-		sfc_MULTILINESTRING = st_multilinestring(),
-		sfc_POLYGON = st_polygon(),
-		sfc_MULTIPOLYGON = st_multipolygon(),
-		st_geometrycollection())
+		sfc_POINT = st_point(rep(NA_real_, ncol), dim = dim),
+		sfc_MULTIPOINT = st_multipoint(matrix(numeric(0), ncol = ncol), dim = dim),
+		sfc_LINESTRING = st_linestring(matrix(numeric(0), ncol = ncol), dim = dim),
+		sfc_MULTILINESTRING = st_multilinestring(dim = dim),
+		sfc_POLYGON = st_polygon(dim = dim),
+		sfc_MULTIPOLYGON = st_multipolygon(dim = dim),
+		st_geometrycollection(dims = dim))
 }
 
 #' retrieve coordinates in matrix form
@@ -432,7 +490,7 @@ st_coordinates.sfc = function(x, ...) {
 
 	ret = switch(class(x)[1],
 		sfc_POINT = matrix(unlist(x, use.names = FALSE), nrow = length(x), byrow = TRUE,
-		     dimnames = list(1:length(x))),
+		     dimnames = list(seq_along(x))),
 		sfc_MULTIPOINT = ,
 		sfc_LINESTRING = coord_2(x),
 		sfc_MULTILINESTRING = ,
@@ -475,7 +533,13 @@ check_ring_dir = function(x) {
 		pol[revert] = lapply(pol[revert], function(m) m[nrow(m):1,])
 		pol
 	}
-	ret = switch(class(x)[1],
+	cls = if (inherits(x, "sfg"))
+			class(x)[2]
+		else
+			class(x)[1]
+	ret = switch(cls,
+		POLYGON = check_polygon(x),
+		MULTIPOLYGON = ,
 		sfc_POLYGON = lapply(x, check_polygon),
 		sfc_MULTIPOLYGON = lapply(x, function(y) structure(lapply(y, check_polygon), class = class(y))),
 		stop(paste("check_ring_dir: not supported for class", class(x)[1]))
