@@ -1,11 +1,33 @@
 ## dplyr methods:
-group_map.sf <- function(.tbl, .f, ...) {
-	 st_as_sf(NextMethod()) # nocov
+#group_map.sf <- function(.tbl, .f, ...) {
+#	 st_as_sf(NextMethod()) # nocov
+#}
+
+# This is currently only used in `bind_rows()` and `bind_cols()`
+# because sf overrides all default implementations
+dplyr_reconstruct.sf = function(data, template) {
+	sfc_name = attr(template, "sf_column")
+	if (inherits(template, "tbl_df"))
+		data = dplyr::as_tibble(data)
+
+	# Return a bare data frame is the geometry column is no longer there
+	if (!sfc_name %in% names(data))
+		return(data)
+
+	prec = st_precision(template)
+	crs = st_crs(template)
+
+	st_as_sf(
+		data,
+		sf_column_name = sfc_name,
+		crs = crs,
+		precision = prec
+	)
 }
 
-group_split.sf <- function(.tbl, ..., keep = TRUE) {
+group_split.sf <- function(.tbl, ..., .keep = TRUE) {
 	 class(.tbl) = setdiff(class(.tbl), "sf")
-     lapply(dplyr::group_split(.tbl, ..., keep = keep), st_as_sf)
+     lapply(dplyr::group_split(.tbl, ..., .keep = .keep), st_as_sf)
 }
 
 #' Tidyverse methods for sf objects (remove .sf suffix!)
@@ -32,7 +54,9 @@ filter.sf <- function(.data, ..., .dots) {
 #' nc %>% select(AREA) %>% arrange(AREA) %>% slice(1:10) %>% plot(add = TRUE, col = 'grey')
 #' title("the ten counties with smallest area")
 arrange.sf <- function(.data, ..., .dots) {
-	st_as_sf(NextMethod(), sf_column_name = attr(.data, "sf_column"))
+	sf_column_name = attr(.data, "sf_column")
+	class(.data) = setdiff(class(.data), "sf")
+	st_as_sf(NextMethod(), sf_column_name = sf_column_name)
 }
 
 #' @name tidyverse
@@ -41,25 +65,38 @@ arrange.sf <- function(.data, ..., .dots) {
 #' nc$area_cl = cut(nc$AREA, c(0, .1, .12, .15, .25))
 #' nc %>% group_by(area_cl) %>% class()
 group_by.sf <- function(.data, ..., add = FALSE) {
+	sf_column_name = attr(.data, "sf_column")
 	class(.data) <- setdiff(class(.data), "sf")
-	st_as_sf(NextMethod(), sf_column_name = attr(.data, "sf_column"))
+	st_as_sf(NextMethod(), sf_column_name = sf_column_name)
 }
 
 #' @name tidyverse
 ungroup.sf <- function(x, ...) {
+	sf_column_name = attr(x, "sf_column")
 	class(x) <- setdiff(class(x), "sf")
-	st_as_sf(NextMethod(), sf_column_name = attr(x, "sf_column"))
+	st_as_sf(NextMethod(), sf_column_name = sf_column_name)
 }
 
-.re_sf = function(x, sf_column_name, agr) {
+#' @name tidyverse
+rowwise.sf <- function(x, ...) {
+	sf_column_name = attr(x, "sf_column")
+	class(x) <- setdiff(class(x), "sf")
+	st_as_sf(NextMethod(), sf_column_name = sf_column_name)
+}
+
+.re_sf = function(x, sf_column_name, agr, geom = NULL) {
 	stopifnot(!inherits(x, "sf"), !missing(sf_column_name), !missing(agr))
 	# non-geom attribute names
 	att = names(x)[!sapply(x, inherits, what = "sfc")]
 	agr = setNames(agr[att], att) # NA's new columns
-	structure(x, 
+	if (!is.null(geom)) {
+		stopifnot(length(geom) == nrow(x))
+		x[[ sf_column_name ]] = geom
+	}
+	structure(x,
 		sf_column = sf_column_name,
 		agr = agr,
-		class = c("sf", class(x))) 
+		class = c("sf", class(x)))
 }
 
 
@@ -69,8 +106,9 @@ ungroup.sf <- function(x, ...) {
 mutate.sf <- function(.data, ..., .dots) {
 	#st_as_sf(NextMethod(), sf_column_name = attr(.data, "sf_column"))
 	agr = st_agr(.data)
+	sf_column_name = attr(.data, "sf_column")
 	class(.data) <- setdiff(class(.data), "sf")
-	.re_sf(NextMethod(), sf_column_name = attr(.data, "sf_column"), agr)
+	.re_sf(NextMethod(), sf_column_name = sf_column_name, agr)
 }
 
 #' @name tidyverse
@@ -78,12 +116,11 @@ mutate.sf <- function(.data, ..., .dots) {
 #' nc %>% transmute(AREA = AREA/10, geometry = geometry) %>% class()
 #' nc %>% transmute(AREA = AREA/10) %>% class()
 transmute.sf <- function(.data, ..., .dots) {
-	ret = NextMethod()
-	if (attr(ret, "sf_column") %in% names(ret))
-		.re_sf(structure(ret, class = setdiff(class(ret), "sf")), # un-sf'd
-			sf_column_name = attr(.data, "sf_column"), st_agr(.data))
-	else
-		ret
+	sf_column_name = attr(.data, "sf_column")
+	agr = st_agr(.data)
+	geom = st_geometry(.data)
+	class(.data) = setdiff(class(.data), "sf")
+	.re_sf(NextMethod(), sf_column_name = sf_column_name, agr, geom)
 }
 
 #' @name tidyverse
@@ -95,19 +132,37 @@ transmute.sf <- function(.data, ..., .dots) {
 #' @details \code{select} keeps the geometry regardless whether it is selected or not; to deselect it, first pipe through \code{as.data.frame} to let dplyr's own \code{select} drop it.
 select.sf <- function(.data, ...) {
 
-	if (!requireNamespace("dplyr", quietly = TRUE))
-		stop("dplyr required: install that first") # nocov
+	if (!requireNamespace("tidyselect", quietly = TRUE))
+		stop("tidyselect required: install that first") # nocov
+	loc = tidyselect::eval_select(quote(c(...)), .data)
 
-	agr <- st_agr(.data)
-	class(.data) <- setdiff(class(.data), "sf")
-	sf_column <- attr(.data, "sf_column")
+	sf_column = attr(.data, "sf_column")
+	sf_column_loc = match(sf_column, names(.data))
 
-	if (!requireNamespace("rlang", quietly = TRUE))
-		stop("rlang required: install first?")
+	if (length(sf_column_loc) != 1 || is.na(sf_column_loc))
+		stop("internal error: can't find sf column") # nocov
 
-	ret <- dplyr::select(.data, ..., !! rlang::sym(sf_column))
-	vars <- setdiff(names(ret), sf_column)
-	st_set_agr(st_as_sf(ret, sf_column_name = sf_column), agr[vars])
+	agr = st_agr(.data)
+	vars = names(.data)[setdiff(loc, sf_column_loc)]
+
+	sf_column_loc_loc = match(sf_column_loc, loc)
+	if (is.na(sf_column_loc_loc)) {
+		# The sf column was subsetted out, select it back in
+		new_agr = setNames(agr[vars], names(loc))
+		loc = c(loc, sf_column_loc)
+		names(loc)[[length(loc)]] = sf_column
+	} else {
+		# The sf column was not subsetted out but it might have been renamed
+		sf_column = names(loc[sf_column_loc_loc])
+		new_agr = setNames(agr[vars], setdiff(names(loc), sf_column))
+	}
+
+	ret = .data
+	class(ret) = setdiff(class(ret), "sf")
+	ret = ret[loc]
+	names(ret) = names(loc)
+
+	st_set_agr(st_as_sf(ret, sf_column_name = sf_column), new_agr)
 }
 
 
@@ -116,27 +171,53 @@ select.sf <- function(.data, ...) {
 #' nc2 <- nc %>% rename(area = AREA)
 rename.sf <- function(.data, ...) {
 
-	if (!requireNamespace("dplyr", quietly = TRUE))
-		stop("dplyr required: install that first") # nocov
+	if (!requireNamespace("tidyselect", quietly = TRUE))
+		stop("tidyselect required: install that first") # nocov
+	loc = tidyselect::eval_rename(quote(c(...)), .data)
 
-	class(.data) <- setdiff(class(.data), "sf")
-	st_as_sf(dplyr::rename(.data, ...))
+	sf_column = attr(.data, "sf_column")
+	sf_column_loc = match(sf_column, names(.data))
+
+	if (length(sf_column_loc) != 1 || is.na(sf_column_loc))
+		stop("internal error: can't find sf column") # nocov
+
+	agr = st_agr(.data)
+	agr_loc = match(names(agr), setdiff(names(.data), sf_column))
+
+	if (anyNA(agr_loc))
+		stop("internal error: can't find `agr` columns") # nocov
+
+	vars_loc = loc[loc %in% agr_loc]
+	names(agr)[vars_loc] = names(vars_loc)
+
+	sf_column_loc_loc = match(sf_column_loc, loc)
+	if (!is.na(sf_column_loc_loc))
+		sf_column = names(loc[sf_column_loc_loc])
+
+	ret = .data
+	class(ret) = setdiff(class(ret), "sf")
+	names(ret)[loc] = names(loc)
+
+	st_set_agr(st_as_sf(ret, sf_column_name = sf_column), agr)
 }
 
 #' @name tidyverse
 #' @examples
 #' nc %>% slice(1:2)
 slice.sf <- function(.data, ..., .dots) {
-	st_as_sf(NextMethod(), sf_column_name = attr(.data, "sf_column"))
+	class(.data) <- setdiff(class(.data), "sf")
+	sf_column <- attr(.data, "sf_column")
+	st_as_sf(NextMethod(), sf_column_name = sf_column)
 }
 
 #' @name tidyverse
 #' @aliases summarise
 #' @param do_union logical; in case \code{summary} does not create a geometry column, should geometries be created by unioning using \link{st_union}, or simply by combining using \link{st_combine}? Using \link{st_union} resolves internal boundaries, but in case of unioning points, this will likely change the order of the points; see Details.
+#' @param is_coverage logical; if \code{do_union} is \code{TRUE}, use an optimized algorithm for features that form a polygonal coverage (have no overlaps)
 #' @return an object of class \link{sf}
-#' @details 
+#' @details
 #' In case one or more of the arguments (expressions) in the \code{summarise} call creates a geometry list-column, the first of these will be the (active) geometry of the returned object. If this is not the case, a geometry column is created, depending on the value of \code{do_union}.
-#' 
+#'
 #' In case \code{do_union} is \code{FALSE}, \code{summarise} will simply combine geometries using \link{c.sfg}. When polygons sharing a boundary are combined, this leads to geometries that are invalid; see for instance \url{https://github.com/r-spatial/sf/issues/681}.
 #' @examples
 #' nc$area_cl = cut(nc$AREA, c(0, .1, .12, .15, .25))
@@ -144,8 +225,12 @@ slice.sf <- function(.data, ..., .dots) {
 #' nc.g %>% summarise(mean(AREA))
 #' nc.g %>% summarise(mean(AREA)) %>% plot(col = grey(3:6 / 7))
 #' nc %>% as.data.frame %>% summarise(mean(AREA))
-summarise.sf <- function(.data, ..., .dots, do_union = TRUE) {
+summarise.sf <- function(.data, ..., .dots, do_union = TRUE, is_coverage = FALSE) {
 	sf_column = attr(.data, "sf_column")
+	precision = st_precision(.data)
+	crs = st_crs(.data)
+	geom = st_geometry(.data)
+	class(.data) = setdiff(class(.data), "sf")
 	ret = NextMethod()
 	if (!missing(do_union))
 		ret$do_union = NULL
@@ -155,25 +240,25 @@ summarise.sf <- function(.data, ..., .dots, do_union = TRUE) {
 				if (!requireNamespace("dplyr", quietly = TRUE))
 					stop("dplyr required: install that first") # nocov
 				i = dplyr::group_indices(.data)
-				geom = st_geometry(.data)
+				# geom = st_geometry(.data)
 				geom = if (do_union)
-						lapply(sort(unique(i)), function(x) st_union(geom[i == x]))
+						lapply(sort(unique(i)), function(x) st_union(geom[i == x], is_coverage = is_coverage))
 					else
 						lapply(sort(unique(i)), function(x) st_combine(geom[i == x]))
 				geom = unlist(geom, recursive = FALSE)
 				if (is.null(geom))
 					geom = list() #676 #nocov
-				do.call(st_sfc, c(geom, crs = list(st_crs(.data)), precision = st_precision(.data)))
+				do.call(st_sfc, c(geom, crs = list(crs), precision = precision))
 			} else { # single group:
 				if (do_union)
-					st_union(st_geometry(.data))
+					st_union(geom, is_coverage = is_coverage)
 				else
-					st_combine(st_geometry(.data))
+					st_combine(geom)
 			}
 		ret[[ sf_column ]] = geom
 	}
 	# need to re-sort out the geometry column class now:
-	st_as_sf(structure(ret, class = setdiff(class(ret), "sf"), "sf_column" = NULL))
+	st_as_sf(structure(ret, sf_column = NULL))
 }
 
 
@@ -185,7 +270,10 @@ summarise.sf <- function(.data, ..., .dots, do_union = TRUE) {
 distinct.sf <- function(.data, ..., .keep_all = FALSE) {
 	sf_column = attr(.data, "sf_column")
 	geom = st_geometry(.data)
-	.data[[ sf_column ]] = vapply(st_equals(.data), head, NA_integer_, n = 1)
+	eq = sapply(st_equals(.data), head, n = 1)
+	empties = which(lengths(eq) == 0)
+	eq[ empties ] = empties[1] # first empty record
+	.data[[ sf_column ]] = unlist(eq)
 	class(.data) = setdiff(class(.data), "sf")
 
 	if (!requireNamespace("dplyr", quietly = TRUE))
@@ -282,7 +370,9 @@ nest.sf = function (.data, ...) {
 
 	class(.data) <- setdiff(class(.data), "sf")
 	ret = tidyr::nest(.data, ...)
-	ret[["data"]] = lapply(ret[["data"]], st_as_sf, sf_column_name = attr(.data, "sf_column"))
+	lst = which(sapply(ret, inherits, "list"))[1]
+	# re-sf:
+	ret[[lst]] = lapply(ret[[lst]], st_as_sf, sf_column_name = attr(.data, "sf_column"))
 	ret
 }
 
@@ -330,14 +420,14 @@ unite.sf <- function(data, col, ..., sep = "_", remove = TRUE) {
 }
 
 #' @name tidyverse
-#' @param .preserve see \link[tidyr]{unnest}
+#' @param .preserve see \link[tidyr:nest]{unnest}
 unnest.sf = function(data, ..., .preserve = NULL) {
 	# nocov start
+	sf_column_name = attr(data, "sf_column", exact = TRUE)
 	if (!requireNamespace("tidyr", quietly = TRUE))
 		stop("unnest requires tidyr; install that first")
-
 	class(data) = setdiff(class(data), "sf")
-	st_sf(NextMethod(), sf_column_name = attr(data, "sf_column", exact = TRUE))
+	st_sf(NextMethod(), sf_column_name = sf_column_name)
 	# nocov end
 }
 
@@ -377,24 +467,29 @@ pillar_shaft.sfc <- function(x, ...) {
 	pillar::new_pillar_shaft_simple(out, align = "right", min_width = 25)
 }
 
-vec_proxy.sfc <- function(x, ...) {
-	x
-}
-
+#nocov start
 register_all_s3_methods = function() {
+	has_dplyr_1.0 =
+		requireNamespace("dplyr", quietly = TRUE) &&
+		utils::packageVersion("dplyr") >= "0.8.99.9000"
+
+	if (has_dplyr_1.0)
+		register_s3_method("dplyr", "dplyr_reconstruct", "sf")
+
 	register_s3_method("dplyr", "anti_join", "sf")
 	register_s3_method("dplyr", "arrange", "sf")
 	register_s3_method("dplyr", "distinct", "sf")
 	register_s3_method("dplyr", "filter", "sf")
 	register_s3_method("dplyr", "full_join", "sf")
 	register_s3_method("dplyr", "group_by", "sf")
-	register_s3_method("dplyr", "group_map", "sf")
+#	register_s3_method("dplyr", "group_map", "sf")
 	register_s3_method("dplyr", "group_split", "sf")
 	register_s3_method("dplyr", "inner_join", "sf")
 	register_s3_method("dplyr", "left_join", "sf")
 	register_s3_method("dplyr", "mutate", "sf")
 	register_s3_method("dplyr", "rename", "sf")
 	register_s3_method("dplyr", "right_join", "sf")
+	register_s3_method("dplyr", "rowwise", "sf")
 	register_s3_method("dplyr", "sample_frac", "sf")
 	register_s3_method("dplyr", "sample_n", "sf")
 	register_s3_method("dplyr", "select", "sf")
@@ -413,12 +508,27 @@ register_all_s3_methods = function() {
 	register_s3_method("pillar", "obj_sum", "sfc")
 	register_s3_method("pillar", "type_sum", "sfc")
 	register_s3_method("pillar", "pillar_shaft", "sfc")
-	register_s3_method("vctrs", "vec_proxy", "sfc")
+	register_s3_method("spatstat", "as.ppp", "sfc")
+	register_s3_method("spatstat", "as.ppp", "sf")
+	register_s3_method("spatstat", "as.owin", "POLYGON")
+	register_s3_method("spatstat", "as.owin", "MULTIPOLYGON")
+	register_s3_method("spatstat", "as.owin", "sfc_POLYGON")
+	register_s3_method("spatstat", "as.owin", "sfc_MULTIPOLYGON")
+	register_s3_method("spatstat", "as.owin", "sfc")
+	register_s3_method("spatstat", "as.owin", "sf")
+	register_s3_method("spatstat", "as.psp", "LINESTRING")
+	register_s3_method("spatstat", "as.psp", "MULTILINESTRING")
+	register_s3_method("spatstat", "as.psp", "sfc_MULTILINESTRING")
+	register_s3_method("spatstat", "as.psp", "sfc")
+	register_s3_method("spatstat", "as.psp", "sf")
+	register_s3_method("wk", "as_wkb", "sf")
+	register_s3_method("wk", "as_wkb", "sfc")
+	register_s3_method("wk", "as_wkb", "sfg")
+	register_vctrs_methods()
 }
 
 # from: https://github.com/tidyverse/hms/blob/master/R/zzz.R
 # Thu Apr 19 10:53:24 CEST 2018
-#nocov start
 register_s3_method <- function(pkg, generic, class, fun = NULL) {
   stopifnot(is.character(pkg), length(pkg) == 1)
   stopifnot(is.character(generic), length(generic) == 1)
