@@ -5,6 +5,9 @@
 # if GEOS_VERSION_MINOR >= 5
 #  define HAVE350
 # endif
+# if GEOS_VERSION_MINOR >= 8
+#  define HAVE380
+# endif
 # if GEOS_VERSION_MINOR == 6
 #  if GEOS_VERSION_PATCH >= 1
 #   define HAVE361
@@ -19,6 +22,7 @@
 #  define HAVE350
 #  define HAVE370
 #  define HAVE361
+#  define HAVE380
 # endif
 #endif
 
@@ -491,6 +495,25 @@ Rcpp::CharacterVector CPL_geos_is_valid_reason(Rcpp::List sfc) {
 	return out;
 }
 
+// #nocov start - no GEOS 3.8.0 on travis yet
+// [[Rcpp::export]]
+Rcpp::List CPL_geos_make_valid(Rcpp::List sfc) {
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+
+	std::vector<GeomPtr> gmv = geometries_from_sfc(hGEOSCtxt, sfc, NULL);
+	std::vector<GeomPtr> out(gmv.size());
+#ifdef HAVE380
+	for (size_t i = 0; i < gmv.size(); i++)
+		gmv[i] = geos_ptr(GEOSMakeValid_r(hGEOSCtxt, gmv[i].get()), hGEOSCtxt);
+#else
+	Rcpp::stop("this shouldn't happen: st_make_valid should use lwgeom");
+#endif
+	Rcpp::List ret = sfc_from_geometry(hGEOSCtxt, gmv);
+	CPL_geos_finish(hGEOSCtxt);
+	return ret;
+}
+// #nocov end
+
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_geos_is_valid(Rcpp::List sfc, bool NA_on_exception = true) {
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
@@ -564,7 +587,14 @@ Rcpp::List CPL_geos_normalize(Rcpp::List sfc) { // #nocov start
 } // #nocov end
 
 // [[Rcpp::export]]
-Rcpp::List CPL_geos_union(Rcpp::List sfc, bool by_feature = false) {
+Rcpp::List CPL_geos_union(Rcpp::List sfc, bool by_feature = false, bool is_coverage = false) {
+
+#ifndef HAVE380
+	if (is_coverage) {
+		Rcpp::warning("ignoring 'is_coverage = TRUE' which requires GEOS version 3.8 or greater");
+		is_coverage = false;
+	}
+#endif
 
 	if (sfc.size() == 0)
 		return sfc; // #nocov
@@ -593,6 +623,12 @@ Rcpp::List CPL_geos_union(Rcpp::List sfc, bool by_feature = false) {
 			gmv_out[0] = std::move(gmv[0]);
 		} else {
 			GeomPtr gc = geos_ptr(GEOSGeom_createCollection_r(hGEOSCtxt, GEOS_GEOMETRYCOLLECTION, to_raw(gmv).data(), gmv.size()), hGEOSCtxt);
+
+#ifdef HAVE380
+			if (is_coverage)
+				gmv_out[0] = geos_ptr(GEOSCoverageUnion_r(hGEOSCtxt, gc.get()), hGEOSCtxt);
+			else
+#endif
 			gmv_out[0] = geos_ptr(GEOSUnaryUnion_r(hGEOSCtxt, gc.get()), hGEOSCtxt);
 		}
 	}
@@ -642,7 +678,8 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
                        Rcpp::NumericVector bufferDist, Rcpp::IntegerVector nQuadSegs,
                        Rcpp::NumericVector dTolerance, Rcpp::LogicalVector preserveTopology,
                        int bOnlyEdges = 1,
-                       Rcpp::IntegerVector endCapStyle = 0, Rcpp::IntegerVector joinStyle = 0, Rcpp::NumericVector mitreLimit = 1)
+                       Rcpp::IntegerVector endCapStyle = 0, Rcpp::IntegerVector joinStyle = 0,
+					   Rcpp::NumericVector mitreLimit = 1, Rcpp::LogicalVector singleside = 0)
 {
 	int dim = 2;
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
@@ -656,8 +693,20 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = geos_ptr(chkNULL(GEOSBuffer_r(hGEOSCtxt, g[i].get(), bufferDist[i], nQuadSegs[i])), hGEOSCtxt);
 	} else if (op == "buffer_with_style") {
-		for (size_t i = 0; i < g.size(); i++)
-			out[i] = geos_ptr(chkNULL(GEOSBufferWithStyle_r(hGEOSCtxt, g[i].get(), bufferDist[i], nQuadSegs[i], endCapStyle[i], joinStyle[i], mitreLimit[i])), hGEOSCtxt);
+		GEOSBufferParams *bufferparams = GEOSBufferParams_create_r(hGEOSCtxt);
+		for (size_t i = 0; i < g.size(); i++) {
+			if (GEOSBufferParams_setEndCapStyle_r(hGEOSCtxt, bufferparams, endCapStyle[i]) &&
+					GEOSBufferParams_setJoinStyle_r(hGEOSCtxt, bufferparams, joinStyle[i]) &&
+					GEOSBufferParams_setMitreLimit_r(hGEOSCtxt, bufferparams, mitreLimit[i]) &&
+					GEOSBufferParams_setQuadrantSegments_r(hGEOSCtxt, bufferparams, nQuadSegs[i]) &&
+					GEOSBufferParams_setSingleSided_r(hGEOSCtxt, bufferparams, singleside[i]))
+				// out[i] = geos_ptr(chkNULL(GEOSBufferWithStyle_r(hGEOSCtxt, g[i].get(), bufferDist[i], nQuadSegs[i], endCapStyle[i], joinStyle[i], mitreLimit[i])), hGEOSCtxt);
+				out[i] = geos_ptr(chkNULL(GEOSBufferWithParams_r(hGEOSCtxt, g[i].get(),
+					bufferparams, bufferDist[i])), hGEOSCtxt);
+			else
+				Rcpp::stop("invalid buffer parameters"); // #nocov
+		}
+		GEOSBufferParams_destroy_r(hGEOSCtxt, bufferparams);
 	} else if (op == "boundary") {
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = geos_ptr(chkNULL(GEOSBoundary_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
@@ -682,7 +731,15 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 		for (size_t i = 0; i < g.size(); i++) {
 			out[i] = geos_ptr(chkNULL(GEOSGetCentroid_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
 		}
-	} else if (op == "node") {
+	} else
+#ifdef HAVE370
+	if (op == "reverse") {
+		for (size_t i = 0; i < g.size(); i++) {
+			out[i] = geos_ptr(chkNULL(GEOSReverse_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
+		}
+	} else
+#endif
+	if (op == "node") {
 		for (size_t i = 0; i < g.size(); i++) {
 			out[i] = geos_ptr(chkNULL(GEOSNode_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
 		}
@@ -697,7 +754,7 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 			out[i] = geos_ptr(chkNULL(GEOSDelaunayTriangulation_r(hGEOSCtxt, g[i].get(), dTolerance[i], bOnlyEdges)), hGEOSCtxt);
 	} else
 #endif
-			Rcpp::stop("invalid operation"); // #nocov
+		Rcpp::stop("invalid operation"); // #nocov
 
 	Rcpp::List ret(sfc_from_geometry(hGEOSCtxt, out, dim));
 	CPL_geos_finish(hGEOSCtxt);
@@ -884,7 +941,6 @@ Rcpp::IntegerVector CPL_geos_nearest_feature(Rcpp::List sfc0, Rcpp::List sfc1) {
 	}
 	Rcpp::IntegerVector out(gmv0.size());
 	for (size_t i = 0; i < gmv0.size(); i++) {
-		out[i] = NA_INTEGER;
 		if (!GEOSisEmpty_r(hGEOSCtxt, gmv0[i].get()) && !tree_is_empty) {
 			item_g item, *ret_item;
 			item.id = 0; // is irrelevant
@@ -896,7 +952,8 @@ Rcpp::IntegerVector CPL_geos_nearest_feature(Rcpp::List sfc0, Rcpp::List sfc1) {
 				out[i] = ret_item->id; // the index (1-based) of nearest GEOM
 			else
 				Rcpp::stop("st_nearest_feature: GEOS exception");
-		}
+		} else
+			out[i] = NA_INTEGER;
 	}
 	CPL_geos_finish(hGEOSCtxt);
 
@@ -1038,6 +1095,14 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 	std::vector<GeomPtr> x = geometries_from_sfc(hGEOSCtxt, sfc, &dim);
 	std::vector<GeomPtr> out;
+	int errors = 0;
+#ifdef HAVE350
+	int notice = 0;
+	GEOSContext_setNoticeMessageHandler_r(hGEOSCtxt,
+		(GEOSMessageHandler_r) __emptyNoticeHandler, (void *) &notice);
+	GEOSContext_setErrorMessageHandler_r(hGEOSCtxt,
+		(GEOSMessageHandler_r) __countErrorHandler, (void *) &notice);
+#endif
 	// initialize trees to find overlapping areas quickly
 	for (size_t i = 0; i < x.size(); i++) {
 		// if i'th geometry in x is empty then skip it
@@ -1050,9 +1115,8 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 				std::vector<size_t> items(out.size());
 				for (size_t j = 0; j < out.size(); j++) {
 					items[j] = j;
-					if (! GEOSisEmpty_r(hGEOSCtxt, out[j].get())) {
+					if (! GEOSisEmpty_r(hGEOSCtxt, out[j].get()))
 						GEOSSTRtree_insert_r(hGEOSCtxt, tree.get(), out[j].get(), &(items[j]));
-					}
 				}
 				// query which geometries in out overlap with geom
 				std::vector<size_t> tree_sel;
@@ -1060,26 +1124,28 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 				// iterate over items in query and erase overlapping areas in geom
 				for (size_t j = 0; j < tree_sel.size(); j++) {
 					size_t k = tree_sel[j];
-					// test if the items are fully contained
-					GeomPtr inters = geos_ptr(GEOSIntersection_r(hGEOSCtxt, out[k].get(), geom.get()), hGEOSCtxt);
-					if (inters == nullptr)
-						Rcpp::stop("GEOS exception"); // #nocov
-					if (! chk_(GEOSisEmpty_r(hGEOSCtxt, inters.get()))) { // i and k intersection
-						geom = geos_ptr(GEOSDifference_r(hGEOSCtxt, geom.get(), inters.get()), hGEOSCtxt); // cut out inters from geom
-						if (geom == nullptr)
-							Rcpp::stop("GEOS exception"); // #nocov
-						GeomPtr g = geos_ptr(GEOSDifference_r(hGEOSCtxt, out[k].get(), inters.get()), hGEOSCtxt); // cut out inters from out[k]
-						if (g == nullptr)
-							Rcpp::stop("GEOS exception"); // #nocov
-						out[k] = std::move(g);
-						out.push_back(std::move(inters)); // keep
-						std::vector<size_t> idx = index[k]; // k < i, and k might already be an intersection
-						idx.push_back(i + 1);
-						index.push_back(idx);
-					}
+					GeomPtr inters = nullptr;
+					if (geom.get() != nullptr) {
+						inters = geos_ptr(GEOSIntersection_r(hGEOSCtxt, out[k].get(), geom.get()), hGEOSCtxt);
+						if (inters == nullptr)
+							errors++;
+						if (inters != nullptr && !chk_(GEOSisEmpty_r(hGEOSCtxt, inters.get()))) { // i and k intersection
+							GeomPtr g = geos_ptr(GEOSDifference_r(hGEOSCtxt, out[k].get(), inters.get()), hGEOSCtxt); // cut out inters from out[k]
+							if (g == nullptr)
+								Rcpp::warning("GEOS difference returns NULL"); // #nocov
+							else {
+								out[k] = std::move(g);
+								out.push_back(std::move(inters)); // keep
+								std::vector<size_t> idx = index[k]; // k < i, and k might already be an intersection
+								idx.push_back(i + 1);
+								index.push_back(idx);
+							}
+						}
+					} else
+						errors++;
 				}
 			}
-			if (! chk_(GEOSisEmpty_r(hGEOSCtxt, geom.get()))) {
+			if (geom != nullptr && ! chk_(GEOSisEmpty_r(hGEOSCtxt, geom.get()))) {
 				out.push_back(std::move(geom));
 				std::vector<size_t> idx;
 				idx.push_back(i + 1);
@@ -1089,6 +1155,14 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 			Rcpp::checkUserInterrupt();
 		}
 	} // for i
+	if (errors > 0)
+		Rcpp::Rcout << "geometry errors: " << errors << std::endl;
+#ifdef HAVE350
+	if (notice > 0)
+		Rcpp::warning("one or more notices ignored");
+	GEOSContext_setNoticeHandler_r(hGEOSCtxt, __warningHandler);
+	GEOSContext_setErrorHandler_r(hGEOSCtxt, __errorHandler);
+#endif
 	size_t j = 0;
 	for (size_t i = 0; i < out.size(); i++) {
 		if (! GEOSisEmpty_r(hGEOSCtxt, out[i].get())) {
